/* Reads Fn keystrokes from /proc/toshiba and handles them
 * Ben Lynn
 */
/*
Copyright (C) 2002 Benjamin Lynn (blynn@cs.stanford.edu)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include <getopt.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>


#include "version.h"
#include "handler.h"

#define DEFAULT_RCFILE "fcrc"
#define DEFAULT_PROCFILE "/proc/toshiba"
#define DEFAULT_CONFDIR "/etc/fncenter/"

char *infile = NULL;

int delay_usec = 50000;

void display_usage()
{
    printf("%s\n", T5005_VERSION);
    printf("Command-line options:\n");
    printf("    -p    choose path containing config files\n");
    printf("    -d    choose /proc file (default is %s)\n", DEFAULT_PROCFILE);
    printf("    -u    specify delay between polls in microseconds (default\
 is 50000)\n");
    printf("    -h    help\n");
    printf("    -V    version\n");
}

int main_loop()
{
    FILE *fp = NULL;
    char s[100];
    int i;

    for(;;) {
	usleep(delay_usec);
	fp = fopen(infile, "r");
	if (fp) {
	    fgets(s, 100, fp);
	    i = strlen(s);
	    i--;
	    s[i] = 0;
	    i-=2;
	    fclose(fp);
	    handle_button(&s[i]);
	}
    }

    return 0;
}

int main(int argc, char **argv)
{
    int c;
    char *s = NULL;
    char *rcfile = NULL;
    char *configpath = NULL;
    FILE *fp = NULL ;  //just for testing whether the config file exists 

    infile = DEFAULT_PROCFILE;

    for(;;) {
	c = getopt(argc, argv, "hVd:u:p:");
	if (c == -1) break;
	switch(c) {
	    case 'h':
		display_usage();
		exit(0);
		break;
	    case 'V':
		printf("Version: %s\n", T5005_VERSION);
		printf("Ben Lynn\n");
		exit(0);
		break;
	    case 'u':
		sscanf(optarg, "%d", &delay_usec);
		break;
	    case 'd':
		infile = optarg;
		break;
	    case 'p':
		configpath = optarg;
		break;
	}
    }
    // this if is true if -p option was specified
    if(configpath)
    {
        rcfile = (char *) malloc(strlen(configpath) + 101);
        if (!rcfile) {
	        perror("malloc");
	        exit(1);
        }
	    strcpy(rcfile, configpath);
	    strcat(rcfile,DEFAULT_RCFILE);
        fp = fopen(rcfile, "r");
        if (fp) {
            fclose(fp);
            handler_init(configpath, rcfile);
            return main_loop();
        }
    }
    
    //now try the conf file in users home dir
    s = getenv("HOME");
	if (s) {
	      if(!configpath) configpath = (char *) malloc(strlen(s) + 101);
	      
          if (!configpath) {
	          perror("malloc");
	          exit(1);
	      }
          if (!rcfile) rcfile = (char *) malloc(strlen(configpath) + 101);
          if (!rcfile) {
	           perror("malloc");
	           exit(1);
          }
	      strcpy(configpath, s);
	      strcat(configpath, "/.fncenter/");
          strcpy(rcfile,configpath);
          strcat(rcfile,DEFAULT_RCFILE);
          fp = fopen("/home/sabathius/.fncenter/fcrc", "r");
//#fp = fopen(rcfile, "r");
          if (fp) {
               fclose(fp);
               handler_init(configpath, rcfile);
               return main_loop();
          }
    }
    //ok, we are left with the default config file
    if (!rcfile) rcfile = (char *) malloc(strlen(configpath) + 101);
    if (!rcfile) {
	      perror("malloc");
	      exit(1);
    }
    strcpy(rcfile, DEFAULT_CONFDIR);
    strcat(rcfile, DEFAULT_RCFILE);
    handler_init(DEFAULT_CONFDIR, rcfile);
    return main_loop();
}
