/* Handler for fncenter
 * Ben Lynn
 */
/*
Copyright (C) 2002 Benjamin Lynn (blynn@cs.stanford.edu)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>

//for open()
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

//for signal()
#include <signal.h>

//for wait()
#include <sys/wait.h>

#include "handler.h"

enum {
    ACTION_PIPE,
    ACTION_FILE,
    ACTION_DIR
};

struct action_s {
    int action;
    char *name;
};

static struct action_s action_list[100];
static int action_count;
static int action_current;

static void sig_chld(int signo)
{
    pid_t pid;
    int stat;

    while ((pid = waitpid(-1, &stat, WNOHANG)) > 0) {
	printf("child %d terminated\n", pid);
    }
    return;
}

static void do_pipe(char *pipefile, char *button)
{
    char s[100];
    int pipefd;

    pipefd = open(pipefile, O_WRONLY | O_NONBLOCK);
    if (pipefd == -1) {
	perror("open");
	return;
    }

    sprintf(s, "%s\n", button);
    write(pipefd, s, strlen(s));
}

static void do_dir(char *dir, char *button)
{
    char name[1024];
    struct stat buf;

    strcpy(name, dir);
    strcat(name, "/");
    strcat(name, button);
    
    if (stat(name, &buf)) {
	return;
    }

    if (!fork()) {
	execl(name, name, NULL);
	perror("exec");
    }
}

static void do_arg(char *file, char *button)
{
    struct stat buf;

    if (stat(file, &buf)) {
	return;
    }

    if (!fork()) {
	execl(file, file, button, NULL);
	perror("exec");
    }
}

static char blast[2] = "00";

void handle_button(char *bname)
{
    struct action_s *a;

    if (!strcmp(blast, bname)) return;
    strcpy(blast, bname);
    if (!strcmp(bname, "00")) return;
    if (!strcmp(bname, "ff")) return;

    printf("%s pressed\n", bname);
    /*
    if (!strcmp(bname, "start")) {
	action_current = (action_current + 1) % action_count;
	return;
    }
    */

    a = &action_list[action_current];
    //printf("current action: %s\n", a->name);
    switch(a->action) {
	case ACTION_PIPE:
	    do_pipe(a->name, bname);
	    break;
	case ACTION_DIR:
	    do_dir(a->name, bname);
	    break;
	case ACTION_FILE:
	    do_arg(a->name, bname);
	    break;
	//default:
    }
}

char *clonepath(char *dir, char *src)
{
    char *dst;

    if (src[0] != '/' && dir) {
	dst = (char *) malloc(strlen(dir) + strlen(src) + 1);
	if (!dst) {
	    return NULL;
	}
	strcpy(dst, dir);
	strcat(dst, src);
    } else {
	dst = (char *) malloc(strlen(src) + 1);
	if (!dst) {
	    return NULL;
	}
	strcpy(dst, src);
    }
    return dst;
}

void handler_init(char *configpath, char *rcfile)
{
    FILE *fp;
    char buf[101];
    char arg[101];

    int i;

    fp = fopen(rcfile, "r");
    if (!fp) {
	fprintf(stderr, "Can't open config file: %s\n", rcfile);
	exit(1);
    }
    fprintf(stdout, "Using: %s\n", rcfile);
    i = 0;
    for(;;) {
	//TODO: buffer overflow?
	fscanf(fp, "%s", buf);
	if (feof(fp)) break;
	fscanf(fp, "%s", arg);
	if (feof(fp)) break;

	if (!strcmp(buf, "script")) {
	    action_list[i].name = clonepath(configpath, arg);
	    action_list[i].action = ACTION_FILE;
	    i++;
	}
	if (!strcmp(buf, "directory")) {
	    action_list[i].name = clonepath(configpath, arg);
	    action_list[i].action = ACTION_DIR;
	    i++;
	}
	if (!strcmp(buf, "pipe")) {
	    action_list[i].name = clonepath(configpath, arg);
	    action_list[i].action = ACTION_PIPE;
	    i++;
	}
    }
    action_count = i;

    fclose(fp);

    signal(SIGCHLD, sig_chld);
}
